/*****************************************************************/
/* IOUTES.C                                                      */
/* I/O Utilities                                                 */
/* (c) 1998 Emmenjay Consulting Pty Ltd                          */
/*                                                               */
/* History                                                       */
/* 05/05/98 MJS  Initial Coding.                                 */
/*                                                               */
/*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "ioutes.h"


void message( char *szFmt, ... )
{
    va_list pArgs;

    /* INIT THE VA_ARGS MACROS */
    va_start( pArgs, szFmt );

    /* OUTPUT THE MESSAGE */
    vprintf( szFmt, pArgs );

    /* CLEAN UP */
    va_end( pArgs );
}

void die( char *szFmt, ... )
{
    va_list pArgs;

    /* INIT THE VA_ARGS MACROS */
    va_start( pArgs, szFmt );

    /* OUTPUT THE MESSAGE */
    vprintf( szFmt, pArgs );

    /* CLEAN UP */
    va_end( pArgs );

    /* ABORT */
    exit( EXIT_FAILURE );
}

char *safecpy( char *dest, char *src, size_t maxlen )
{
    strncpy( dest, src, maxlen );
    dest[maxlen-1] = 0;
    return dest;
}

void ask_val( char *szPrompt, char *szBuff, int iBufflen )
{
    message( szPrompt );
    szBuff[0] = '\0';

    while (!strlen(szBuff)) {
        fgets( szBuff, iBufflen, stdin );
        trimstr( szBuff );
    }
}

char *trimstr( char *szBuff )
{
    char *p;

    p = szBuff + strlen(szBuff) - 1;
    while (p>=szBuff && isspace(*p)) *(p--) = '\0';

    return szBuff;
}

char *makestr( char *szFmt, ... )
{
    va_list pArgs;
    char szBuff[1024];
    char *pszStr;

    va_start( pArgs, szFmt );
    vsprintf( szBuff, szFmt, pArgs );
    va_end( pArgs );

    pszStr = strdup( szBuff );

    if (!pszStr) die( "Out of memory\n" );

    return pszStr;
}


void logfile( char *szFmt, ... )
{
    char szLogfile[40];
    FILE *fLog;
    va_list pArgs;
    time_t tNow;
    struct tm *ptmNow;

    time( &tNow );
    ptmNow = localtime( &tNow );
    sprintf( szLogfile, "%04d-%02d-%02d.log",
             ptmNow->tm_year+1900, ptmNow->tm_mon+1,
             ptmNow->tm_mday );
    fLog = fopen( szLogfile, "ab" );

    if (fLog) {
        va_start( pArgs, szFmt );
        vfprintf( fLog, szFmt, pArgs );
        va_end( pArgs );
        fclose( fLog );
    }        
    
}

char *timestamp( void )
{
    time_t tNow;
    static char szTime[80];
	
	time( &tNow );
	strcpy( szTime, ctime(&tNow) );
    trimstr( szTime );
	return szTime;
}

