/*****************************************************************/
/* SMAIL.C                                                       */
/* Send a mail message                                           */
/* (c) 1998 Emmenjay Consulting Pty Ltd                          */
/*                                                               */
/* History                                                       */
/* 05/05/98 MJS  Initial Coding.                                 */
/*                                                               */
/*****************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <winsock2.h>
#include <time.h>
#include "ioutes.h"
#include "version.h"

#define SZ_PARAM 255


int transmit( SOCKET hServer, char *szFmt, ... );
int tx_send( SOCKET hServer, char *szBuff );
int tx_recv( SOCKET hServer );
void get_params( int argc, char *argv[], unsigned maxlen, 
                 char *szSmtpServerName, char *szToAddr, 
                 char *szFromAddr, char *szMailFile );



int main(int argc, char *argv[])
{
    WSADATA        WSData;
    LPHOSTENT      lpHostEntry;
    LPSERVENT      lpServEntry;
    SOCKADDR_IN    SockAddr;
    SOCKET         hServer;
    int            iProtocolPort;
    char           szServerName[SZ_PARAM];
    char           szToAddr[SZ_PARAM];
    char           szFromAddr[SZ_PARAM];
    char           szMailFile[SZ_PARAM];
    char           szMsgLine[255];
    FILE           *fMsg;
    WORD           wVersion;


	logfile( "\r\nCommencing session at %s\r\n", timestamp() );

    /* PARSE COMMAND LINE PARAMS */
    get_params( argc, argv, SZ_PARAM, 
                szServerName, szToAddr, 
                szFromAddr, szMailFile );

    /* OPEN MAIL MESSAGE FILE */
    fMsg = fopen( szMailFile, "r" );
    if (!fMsg) {
        die( "Unable to open message file \"%s\"\n", 
             szMailFile );
    }

    /* ATTEMPT TO INTIALIZE WINSOCK (1.1 OR LATER). */
    wVersion = MAKEWORD( VERSION_MAJOR, VERSION_MINOR );
    if ( WSAStartup(wVersion, &WSData) ) {
        fclose( fMsg );
        die( "Cannot find Winsock v%d.%d or later\n", 
           VERSION_MAJOR, VERSION_MINOR );
    }

    /* LOOKUP EMAIL SERVER'S IP ADDRESS. */
    lpHostEntry = gethostbyname( szServerName );
    if ( !lpHostEntry ) {
        fclose( fMsg );
        WSACleanup();
        die( "Cannot find SMTP mail server %s\n",
             szServerName );
    }
    
    /* CREATE A TCP/IP SOCKET, NO SPECIFIC PROTOCOL. */
    hServer = socket( PF_INET, SOCK_STREAM, 0); 
    if (hServer == INVALID_SOCKET) {
        fclose( fMsg );
        WSACleanup();
        die( "Cannot open mail server socket\n" );
    }

    /* GET THE MAIL SERVICE PORT.  */
    lpServEntry = getservbyname( "mail", 0 );

    /* USE THE SMTP DEFAULT PORT IF NO OTHER PORT IS SPECIFIED. */
    if (lpServEntry == NULL) {
      iProtocolPort = htons(IPPORT_SMTP);
    } else {
      iProtocolPort = lpServEntry->s_port;
    }

    /* SETUP A SOCKET ADDRESS STRUCTURE. */
    SockAddr.sin_family = AF_INET;
    SockAddr.sin_port = iProtocolPort;
    SockAddr.sin_addr = *((LPIN_ADDR)*lpHostEntry->h_addr_list);

    /* CONNECT THE SOCKET. */
    if (connect( hServer, (PSOCKADDR) &SockAddr, sizeof(SockAddr))) {
        WSACleanup();
        die( "Error connecting to Server socket.\n" );
    }

    /* RECEIVE INITIAL RESPONSE FROM SMTP SERVER. */
    tx_recv( hServer );

	/* TRANSMIT HEADER INFO */
    transmit( hServer, "HELO %s\r\n", szServerName );
    transmit( hServer, "MAIL FROM:<%s>\r\n", szFromAddr );
    transmit( hServer, "RCPT TO:<%s>\r\n", szToAddr );
    transmit( hServer, "DATA\r\n" );
    

    /* SEND MESSAGE BODY */
    while (fgets( szMsgLine, sizeof(szMsgLine)-2, fMsg )) {
        trimstr( szMsgLine );
        strcat( szMsgLine, "\r\n" );
        tx_send( hServer, szMsgLine );
    }

    /* SEND MESSAGE TRAILER INFO */
    transmit( hServer, "\r\n.\r\n" );
    transmit( hServer, "QUIT\r\n" );


    /* REPORT COMPLETION */
    message( "\n\nSent %s as email message to %s\n", 
             szMailFile, szToAddr );
    
    /* CLOSE SERVER SOCKET AND PREPARE TO EXIT. */
    closesocket( hServer );
    WSACleanup();

	logfile( "Completed session at %s\r\n\r\n\r\n", timestamp() );

    return 0;
}
 

void get_params( int argc, char *argv[], unsigned maxlen, 
                 char *szServerName, char *szToAddr, 
                 char *szFromAddr, char *szMailFile )
{
    if (argc!=5) {
        char *szMsg = 
            "Usage: smail mailserv to_addr from_addr messagefile\n"
            "Example: smail smtp.myisp.com rcvr@elsewhere.com "
            "my_id@mydomain.com message.txt\n";
        die( szMsg );
    }

    safecpy( szServerName, argv[1], maxlen );
    safecpy( szToAddr,     argv[2], maxlen );
    safecpy( szFromAddr,   argv[3], maxlen );
    safecpy( szMailFile,   argv[4], maxlen );
}

int transmit( SOCKET hServer, char *szFmt, ... )
{
    BOOL bOK;
    char szOutBuff[256];
    va_list pArgs;

    /* FORMAT THE OUTPUT STRING */
    va_start( pArgs, szFmt );
    vsprintf( szOutBuff, szFmt, pArgs );
    va_end( pArgs );

	/* SEND THE STRING */
    bOK = tx_send( hServer, szOutBuff );

    /* RECEIVE THE REPLY */
    if (!tx_recv( hServer )) {
        bOK = 0;
    }

    return bOK;
}

int tx_send( SOCKET hServer, char *szBuff )
{
    int iStatus;

    iStatus = send( hServer, szBuff, 
                    strlen(szBuff), 0 );

    logfile( "Tx  : %s", szBuff );

    if (iStatus==SOCKET_ERROR || iStatus==0) {
        logfile( "Err : %d - %d Transmitting.\r\n", 
                 iStatus, GetLastError() );
        return 0;
    } else {
        return 1;
    }
}


int tx_recv( SOCKET hServer )
{
    int iStatus;
    char szBuff[4096];

    memset( szBuff, 0, sizeof(szBuff) );

    iStatus = recv( hServer, szBuff, 
                    sizeof(szBuff), 0 );
    if (iStatus==SOCKET_ERROR || iStatus==0) {
        logfile( "Err : %d - %d Receiving.\r\n", 
                 iStatus, GetLastError() );
        return 0;
    } else {
        logfile( "Rx  : %s\r\n", szBuff );
        return 1;
    }
}
